/*
 * q_ooo.c	Produce out-of-order packets (userspace tc module)
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Authors:	Catalin(ux aka Dino) BOIE, <catab at umbrella dot ro>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "utils.h"
#include "tc_util.h"


/* Temporary, till glibc gets this */
/* ooo section */
struct tc_ooo_qopt
{
	__u32	limit;
	__u32	gap;
	__u32	wait;
};


static void explain(void)
{
	fprintf(stderr, "Usage: ... ooo [gap NUMBER] [wait NUMBER_IN_ms] [limit NUMBER]\n");
}

#define usage() return(-1)

static int ooo_parse_opt(struct qdisc_util *qu, int argc, char **argv, struct nlmsghdr *n)
{
	int ok=0;
	struct tc_ooo_qopt opt;

	memset(&opt, 0, sizeof(opt));

	while (argc > 0) {
		if (strcmp(*argv, "limit") == 0) {
			NEXT_ARG();
			if (get_size(&opt.limit, *argv)) {
				fprintf(stderr, "Illegal \"limit\"\n");
				return -1;
			}
			ok++;
		} else if (strcmp(*argv, "wait") == 0) {
			NEXT_ARG();
			if (get_size(&opt.wait, *argv)) {
				fprintf(stderr, "Illegal \"wait\"\n");
				return -1;
			}
			ok++;
		} else if (strcmp(*argv, "gap") == 0) {
			NEXT_ARG();
			if (get_size(&opt.gap, *argv)) {
				fprintf(stderr, "Illegal \"gap\"\n");
				return -1;
			}
			ok++;
		} else if (strcmp(*argv, "help") == 0) {
			explain();
			return -1;
		} else {
			fprintf(stderr, "What is \"%s\"?\n", *argv);
			explain();
			return -1;
		}
		argc--; argv++;
	}

	if (ok)
		addattr_l(n, 1024, TCA_OPTIONS, &opt, sizeof(opt));
	return 0;
}

static int ooo_print_opt(struct qdisc_util *qu, FILE *f, struct rtattr *opt)
{
	struct tc_ooo_qopt *qopt;

	if (opt == NULL)
		return 0;

	if (RTA_PAYLOAD(opt) < sizeof(*qopt))
		return -1;
	qopt = RTA_DATA(opt);

	fprintf (f, "limit %up gap %up wait %ums",
		qopt->limit, qopt->gap, qopt->wait);
	return 0;
}

static int ooo_print_xstats(struct qdisc_util *qu, FILE *f, struct rtattr *xstats)
{
	return 0;
}


struct qdisc_util ooo_util = {
	NULL,
	"ooo",
	ooo_parse_opt,
	ooo_print_opt,
	ooo_print_xstats,
};
